@@ -10,7 +10,8 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
-# pylint: disable=missing-module-docstring
+
+"""Nobody inspects the spammish repetition."""
 
 from __future__ import annotations
 
