@@ -15,7 +15,7 @@
 
 """The client for the backdoor API of the website."""
 
-from __future__ import annotations, barry_as_FLUFL
+from __future__ import annotations
 
 import ast
 import asyncio
@@ -31,7 +31,7 @@ import traceback
 import uuid
 from collections.abc import Callable, Iterable
 from types import EllipsisType
-from typing import Any
+from typing import Any, TypedDict
 from urllib.parse import SplitResult, quote, quote_plus, urlsplit
 
 try:
@@ -57,11 +57,19 @@ else:
     uvloop.install()
 
 
+FLUFL = True
+
 E = eval(  # pylint: disable=eval-used
     "eval(repr((_:=[],_.append(_))[0]))[0][0]"
 )
 
 
+class Response(TypedDict):  # noqa: D101
+    success: bool | EllipsisType
+    output: None | str
+    result: None | tuple[str, bytes | None] | SystemExit
+
+
 async def create_socket(  # pylint: disable=too-many-arguments  # noqa: C901
     addr: str,
     port: int | str,
@@ -221,11 +229,16 @@ async def request(  # pylint: disable=too-many-branches, too-many-locals  # noqa
 
 def detect_mode(code: str) -> str:
     """Detect which mode needs to be used."""
+    import __future__
+
+    flags: int = ast.PyCF_ONLY_AST
+    if FLUFL:
+        flags |= __future__.barry_as_FLUFL.compiler_flag
     try:
-        ast.parse(code, mode="eval")
+        compile(code, "", "eval", flags, 0x5F3759DF)
         return "eval"
     except SyntaxError:
-        ast.parse(code, mode="exec")
+        compile(code, "", "exec", flags, 0x5F3759DF)
         return "exec"
 
 
@@ -242,21 +255,14 @@ def send(
     proxy_rdns: None | bool = True,
     proxy_username: None | str = None,
     proxy_password: None | str = None,
-) -> tuple[
-    int,
-    dict[str, str],
-    None
-    | str
-    | SystemExit
-    | dict[str, None | bool | str | tuple[str, None | bytes]],
-]:
+) -> tuple[int, dict[str, str], Response | str | None]:
     """Send code to the backdoor API."""
     body = code.encode("utf-8")
     if isinstance(url, str):
         url = urlsplit(url)
-    headers = {
-        "Authorization": key,
-    }
+    headers = {"Authorization": key}
+    if FLUFL:
+        headers["X-Future-Feature"] = "barry_as_FLUFL"
     if session:
         headers["X-Backdoor-Session"] = session
     response = asyncio.run(
@@ -369,13 +375,13 @@ def run_and_print(  # noqa: C901  # pylint: disable=too-many-arguments, too-many
     if isinstance(body, str):
         print("\033[91m" + body + "\033[0m")
         return
-    if isinstance(body, SystemExit):
-        raise body  # pylint: disable=raising-bad-type
     if isinstance(body, dict):
         print(f"Success: {body['success']}")
         if isinstance(body["output"], str) and body["output"]:
             print("Output:")
             print(body["output"].strip())
+        if isinstance(body["result"], SystemExit):
+            raise body["result"]  # pylint: disable=raising-bad-type
         if isinstance(body["result"], tuple):
             if not body["success"]:
                 print(body["result"][0])
@@ -390,7 +396,7 @@ def run_and_print(  # noqa: C901  # pylint: disable=too-many-arguments, too-many
             if (
                 isinstance(result_obj, tuple)
                 and len(result_obj) == 2
-                and result_obj[0] == "HelperTuple"
+                and result_obj[0] == "PagerTuple"
                 and isinstance(result_obj[1], str)
             ):
                 pydoc.pager(result_obj[1])
@@ -405,19 +411,20 @@ def run_and_print(  # noqa: C901  # pylint: disable=too-many-arguments, too-many
 def main() -> None | int | str:  # noqa: C901  # pylint: disable=useless-return
     # pylint: disable=too-complex, too-many-branches
     # pylint: disable=too-many-locals, too-many-statements
-    """Parse arguments, load the cache and start the backdoor client."""
+    """Parse arguments, load the config and start the backdoor client."""
     if "--help" in sys.argv or "-h" in sys.argv:
         sys.exit(
             """
 
 Accepted arguments:
 
-    --clear-cache      clear the whole cache
+    --dev              use a separate config for a local developing instance
     --lisp             enable Lots of Irritating Superfluous Parentheses
-    --new-proxy        don't use the cached proxy
-    --new-session      start a new session with cached URL and key
-    --no-cache         start without a cache
+    --new-proxy        don't use the saved proxy
+    --new-session      start a new session with saved URL and key
+    --no-config        start without loading/saveing the config
     --no-patch-help    don't patch help()
+    --reset-config     reset the whole config
     --timing           print the time it took to execute each command
 
     --help or -h       show this help message
@@ -426,12 +433,13 @@ Accepted arguments:
         )
     for arg in sys.argv[1:]:
         if arg not in {
-            "--clear-cache",
+            "--dev",
             "--lisp",
             "--new-proxy",
             "--new-session",
-            "--no-cache",
+            "--no-config",
             "--no-patch-help",
+            "--reset-config",
             "--timing",
             "--help",
             "-h",
@@ -448,30 +456,31 @@ Accepted arguments:
     proxy_rdns: None | bool = True
     proxy_username: None | str = None
     proxy_password: None | str = None
-    cache_pickle = os.path.join(
-        os.path.expanduser(os.getenv("XDG_CACHE_HOME") or "~/.cache"),
-        "an-backdoor-client/session.pickle",
+    config_pickle = os.path.join(
+        os.path.expanduser(os.getenv("XDG_CONFIG_HOME") or "~/.config"),
+        "an-backdoor-client/"
+        + ("dev-" if "--dev" in sys.argv else "")
+        + "session.pickle",
     )
-    if "--clear-cache" in sys.argv:
-        if os.path.exists(cache_pickle):
-            os.remove(cache_pickle)
-        print("Cache cleared")
-    if "--no-cache" not in sys.argv:
+    if "--reset-config" in sys.argv:
+        if os.path.exists(config_pickle):
+            os.remove(config_pickle)
+    if "--no-config" not in sys.argv:
         try:
-            with open(cache_pickle, "rb") as file:
-                cache = pickle.load(file)
+            with open(config_pickle, "rb") as file:
+                config = pickle.load(file)
         except FileNotFoundError:
             pass
         else:
-            url = cache.get("url")
-            key = cache.get("key")
-            session = cache.get("session")
-            proxy_type = cache.get("proxy_type")
-            proxy_addr = cache.get("proxy_addr")
-            proxy_port = cache.get("proxy_port")
-            proxy_rdns = cache.get("proxy_rdns")
-            proxy_username = cache.get("proxy_username")
-            proxy_password = cache.get("proxy_password")
+            url = config.get("url")
+            key = config.get("key")
+            session = config.get("session")
+            proxy_type = config.get("proxy_type")
+            proxy_addr = config.get("proxy_addr")
+            proxy_port = config.get("proxy_port")
+            proxy_rdns = config.get("proxy_rdns")
+            proxy_username = config.get("proxy_username")
+            proxy_password = config.get("proxy_password")
             if "--new-session" in sys.argv:
                 print(f"Using URL {url}")
             else:
@@ -532,9 +541,9 @@ Accepted arguments:
             session = str(uuid.uuid4())
         print(f"Using session {session}")
 
-    if "--no-cache" not in sys.argv:
-        os.makedirs(os.path.dirname(cache_pickle), exist_ok=True)
-        with open(cache_pickle, "wb") as file:
+    if "--no-config" not in sys.argv:
+        os.makedirs(os.path.dirname(config_pickle), exist_ok=True)
+        with open(config_pickle, "wb") as file:
             pickle.dump(
                 {
                     "url": url,
@@ -549,53 +558,56 @@ Accepted arguments:
                 },
                 file,
             )
-        print("Saved information to cache")
+
+    def send_to_remote(code: str, *, mode: str) -> Any:
+        """Send code to the remote backdoor and return the unpickled body."""
+        return send(
+            url,  # type: ignore[arg-type]
+            key,  # type: ignore[arg-type]
+            code,
+            mode,
+            session,
+            proxy_type=proxy_type,  # type: ignore[arg-type]
+            proxy_addr=proxy_addr,
+            proxy_port=proxy_port,
+            proxy_rdns=proxy_rdns,
+            proxy_username=proxy_username,
+            proxy_password=proxy_password,
+        )[2]
 
     if "--no-patch-help" not in sys.argv:
-        code = (
+        body = send_to_remote(
             "def help(*args, **kwargs):\n"
             "    import io\n"
             "    import pydoc\n"
             "    helper_output = io.StringIO()\n"
             "    pydoc.Helper(io.StringIO(), helper_output)(*args, **kwargs)\n"
-            "    return 'HelperTuple', helper_output.getvalue()"
+            "    return 'PagerTuple', helper_output.getvalue()",
+            mode="exec",
         )
-        response = send(
-            url,
-            key,
-            code,
-            "exec",
-            session,
-            proxy_type=proxy_type,
-            proxy_addr=proxy_addr,
-            proxy_port=proxy_port,
-            proxy_rdns=proxy_rdns,
-            proxy_username=proxy_username,
-            proxy_password=proxy_password,
-        )
-        status, headers, body = response
         if not (isinstance(body, dict) and body["success"]):
             print("\033[91mPatching help() failed!\033[0m")
 
     if "--lisp" in sys.argv:
         if not hy:
-            sys.exit("Hy is not installed!")
-        response = send(
-            url,
-            key,
-            "__import__('hy')",
-            "eval",
-            session,
-            proxy_type=proxy_type,
-            proxy_addr=proxy_addr,
-            proxy_port=proxy_port,
-            proxy_rdns=proxy_rdns,
-            proxy_username=proxy_username,
-            proxy_password=proxy_password,
-        )
-        status, headers, body = response
+            sys.exit("\033[91mHy is not installed!\033[0m")
+        body = send_to_remote("import hy", mode="exec")
         if not (isinstance(body, dict) and body["success"]):
-            print("\033[91mImporting Hy builtins failed!\033[0m")
+            print("\033[91mInjecting Hy builtins failed!\033[0m")
+
+    body = send_to_remote(
+        "import sys\nprint('Python', sys.version, 'on', sys.platform)",
+        mode="exec",
+    )
+    if isinstance(body, dict) and body["success"] and body["output"]:
+        print(f"\033[92mConnection to {url} was successful.\033[0m")
+        print(body["output"].strip())
+    else:
+        print("\033[91mGetting remote information failed.\033[0m")
+    print(
+        'Type "copyright", "credits" or '
+        'use the "help" function for more information.'
+    )
 
     # pylint: disable=import-outside-toplevel
     from pyrepl.python_reader import ReaderConsole  # type: ignore
@@ -629,7 +641,7 @@ Accepted arguments:
 
     # run the reader
     try:
-        _main()
+        _main(print_banner=False)
     except EOFError:
         pass
     return None
