@@ -1,4 +1,4 @@
-// @license magnet:?xt=urn:btih:0b31508aeb0634b347b8270c7bee4d411b5d4109&dn=agpl-3.0.txt AGPL-3.0
+// @license magnet:?xt=urn:btih:0b31508aeb0634b347b8270c7bee4d411b5d4109&dn=agpl-3.0.txt GNU-AGPL-3.0-or-later
 function removeAllPopups() {
     for (let node of d.getElementsByClassName("popup-container")) {
         node.remove();
@@ -58,16 +58,13 @@ function startLoadingComics() {
         [
             getDateBy(2022, 4, 4),
             "administratives/kaenguru-comics/2022-04/4/original"
-        ]
+        ],
+        [
+          getDateBy(2022, 5, 9),
+            "administratives/kaenguru-comics/2022-05/9/original"
+        ],
     ];
 
-    const datesEqual = (date1, date2) => dateEquals(
-        date1,
-        date2.getFullYear(),
-        date2.getMonth() + 1,  // js is stupid
-        date2.getDate()
-    );
-
     const dateEquals = (date, year, month, dayOfMonth) => (
         // check if a date equals another based on year, month, and dayOfMonth
         date.getFullYear() === year
@@ -75,6 +72,13 @@ function startLoadingComics() {
         && date.getDate() === dayOfMonth
     );
 
+    const datesEqual = (date1, date2) => dateEquals(
+        date1,
+        date2.getFullYear(),
+        date2.getMonth() + 1,  // js is stupid
+        date2.getDate()
+    );
+
     const isSunday = (date) => (
         date
         && date.getDay() === 0
