@@ -21,28 +21,44 @@ import importlib
 import logging
 import os
 import re
+import signal
 import ssl
 import sys
+import types
 from collections.abc import Callable, Coroutine
+from multiprocessing import process
 from pathlib import Path
-from typing import Any
+from typing import Any, cast
 
 import orjson
+import tornado.netutil
+import tornado.process
 from ecs_logging import StdlibFormatter
 from elastic_enterprise_search import AppSearch  # type: ignore
 from elasticapm.contrib.tornado import ElasticAPM  # type: ignore
 from elasticsearch import AsyncElasticsearch, NotFoundError
-from redis.asyncio import (  # type: ignore
+from redis.asyncio import (
     BlockingConnectionPool,
     Redis,
     SSLConnection,
     UnixDomainSocketConnection,
 )
+from tornado.httpserver import HTTPServer
 from tornado.log import LogFormatter
 from tornado.web import Application, RedirectHandler, StaticFileHandler
 
-from . import DIR, NAME, TEMPLATES_DIR, VERSION
+from . import (
+    CONTAINERIZED,
+    DIR,
+    EVENT_ELASTICSEARCH,
+    EVENT_REDIS,
+    EVENT_SHUTDOWN,
+    NAME,
+    TEMPLATES_DIR,
+    VERSION,
+)
 from .contact.contact import apply_contact_stuff_to_app
+from .quotes import AUTHORS_CACHE, QUOTES_CACHE, WRONG_QUOTES_CACHE
 from .utils import static_file_handling
 from .utils.request_handler import BaseRequestHandler, NotFoundHandler
 from .utils.utils import Handler, ModuleInfo, Permissions, Timer, time_function
@@ -93,6 +109,7 @@ def get_module_infos() -> str | tuple[ModuleInfo, ...]:
                 f".{module_name}",
                 package="an_website",
             )
+
             if "get_module_info" not in dir(module):
                 errors.append(
                     f"{os.path.join(DIR, potential_module, potential_file)} "
@@ -102,21 +119,12 @@ def get_module_infos() -> str | tuple[ModuleInfo, ...]:
                 )
                 continue
 
-            if (
-                (  # check if the annotations specify the return
-                    # type as Module info
-                    module.get_module_info.__annotations__.get("return", "")
-                    == "ModuleInfo"
-                )
-                # check if returned module_info is type ModuleInfo
-                and isinstance(
-                    module_info := module.get_module_info(),
-                    ModuleInfo,
-                )
+            if isinstance(
+                module_info := module.get_module_info(),
+                ModuleInfo,
             ):
                 module_infos.append(module_info)
                 loaded_modules.append(module_name)
-
                 if import_timer.stop() > 0.1:
                     logger.warning(
                         "Import of %s took %ss. "
@@ -129,9 +137,8 @@ def get_module_infos() -> str | tuple[ModuleInfo, ...]:
             errors.append(
                 "'get_module_info' in "
                 f"{os.path.join(DIR, potential_module, potential_file)} "
-                "does not return ModuleInfo. "
-                "Please add/fix the return type or add "
-                f"'{potential_module}.*' or '{module_name}' to "
+                "does not return ModuleInfo. Please fix the returned value "
+                f"or add '{potential_module}.*' or '{module_name}' to "
                 "IGNORED_MODULES."
             )
 
@@ -143,28 +150,29 @@ def get_module_infos() -> str | tuple[ModuleInfo, ...]:
         logger.error("\n".join(errors))
 
     logger.info(
-        "loaded %d modules: '%s'",
+        "Loaded %d modules: '%s'",
         len(loaded_modules),
         "', '".join(loaded_modules),
     )
+
     logger.info(
-        "ignored %d modules: '%s'",
+        "Ignored %d modules: '%s'",
         len(IGNORED_MODULES),
         "', '".join(IGNORED_MODULES),
     )
 
     sort_module_infos(module_infos)
 
-    # make module_infos immutable so it never changes:
+    # make module_infos immutable so it never changes
     return tuple(module_infos)
 
 
 def sort_module_infos(module_infos: list[ModuleInfo]) -> None:
     """Sort a list of module info and move the main page to the top."""
-    # sort it so the order makes sense.
+    # sort it so the order makes sense
     module_infos.sort()
 
-    # move the main page to the top:
+    # move the main page to the top
     for i, info in enumerate(module_infos):
         if info.path == "/":
             module_infos.insert(0, module_infos.pop(i))
@@ -226,12 +234,11 @@ def get_all_handlers(
         (r"/.well-known/(.*)", StaticFileHandler, {"path": ".well-known"})
     )
 
-    if sys.flags.dev_mode:
-        logger.debug(
-            "loaded %d handlers: %s",
-            len(handlers),
-            "; ".join(str(handler) for handler in handlers),
-        )
+    logger.debug(
+        "Loaded %d handlers: %s",
+        len(handlers),
+        "; ".join(str(handler) for handler in handlers),
+    )
 
     return handlers
 
@@ -248,12 +255,11 @@ def make_app() -> str | Application:
         )
     handlers = get_all_handlers(module_infos)
     return Application(
-        handlers,  # type: ignore
+        handlers,  # type: ignore[arg-type]
         MODULE_INFOS=module_infos,
         HANDLERS=handlers,
         # General settings
         autoreload=False,
-        compress_response=True,
         debug=bool(sys.flags.dev_mode),
         default_handler_class=NotFoundHandler,
         websocket_ping_interval=10,
@@ -278,6 +284,10 @@ def apply_config_to_app(
         "GENERAL", "COOKIE_SECRET", fallback=b"xyzzy"
     )
 
+    app.settings["NETCUP"] = config.getboolean(
+        "GENERAL", "NETCUP", fallback=False
+    )
+
     app.settings["TRUSTED_API_SECRETS"] = {
         key_perms[0]: Permissions(
             int(key_perms[1])
@@ -291,7 +301,6 @@ def apply_config_to_app(
         if key_perms[0]
     }
 
-    # the onion address of this website
     onion_address = config.get("GENERAL", "ONION_ADDRESS", fallback=None)
     app.settings["ONION_ADDRESS"] = onion_address
     if onion_address is None:
@@ -299,12 +308,12 @@ def apply_config_to_app(
     else:
         app.settings["ONION_PROTOCOL"] = onion_address.split("://")[0]
 
-    # whether ratelimits are enabled
     app.settings["RATELIMITS"] = config.getboolean(
         "GENERAL", "RATELIMITS", fallback=False
     )
 
     app.settings["REDIS_PREFIX"] = config.get("REDIS", "PREFIX", fallback=NAME)
+
     app.settings["ELASTICSEARCH_PREFIX"] = config.get(
         "ELASTICSEARCH", "PREFIX", fallback=NAME
     )
@@ -322,15 +331,21 @@ def get_ssl_context(
             config.get("TLS", "PASSWORD", fallback=None),
         )
         return ssl_ctx
-
     return None
 
 
-def setup_logging(config: configparser.ConfigParser) -> None:
+def setup_logging(
+    config: configparser.ConfigParser, *, allow_file_handler: bool = True
+) -> None:
     """Configure logging."""
     debug = config.getboolean("LOGGING", "DEBUG", fallback=sys.flags.dev_mode)
+    path = config.get(
+        "LOGGING", "PATH", fallback=None if sys.flags.dev_mode else "logs"
+    )
+
     root_logger = logging.getLogger()
     root_logger.setLevel(logging.DEBUG if debug else logging.INFO)
+
     stream_handler = logging.StreamHandler(stream=sys.stdout)
     stream_handler.setFormatter(
         logging.Formatter(
@@ -342,10 +357,7 @@ def setup_logging(config: configparser.ConfigParser) -> None:
     )
     root_logger.addHandler(stream_handler)
 
-    path = config.get(
-        "LOGGING", "PATH", fallback=None if sys.flags.dev_mode else "logs"
-    )
-    if path:
+    if allow_file_handler and path:
         os.makedirs(path, 0o755, True)
         file_handler = logging.handlers.TimedRotatingFileHandler(
             os.path.join(path, f"{NAME}.log"),
@@ -421,10 +433,14 @@ def setup_app_search(app: Application) -> None:
     )
 
 
-async def setup_elasticsearch(app: Application, raise_exc: bool = True) -> None:
+def setup_elasticsearch(app: Application) -> None:
     """Setup Elasticsearch."""  # noqa: D401
     config = app.settings["CONFIG"]
-    elasticsearch = AsyncElasticsearch(
+    if not config.getboolean("ELASTICSEARCH", "ENABLED", fallback=False):
+        app.settings["ELASTICSEARCH"] = None
+        return
+
+    app.settings["ELASTICSEARCH"] = AsyncElasticsearch(
         cloud_id=config.get("ELASTICSEARCH", "CLOUD_ID", fallback=None),
         hosts=tuple(
             host.strip()
@@ -455,18 +471,28 @@ async def setup_elasticsearch(app: Application, raise_exc: bool = True) -> None:
             "accept": "application/vnd.elasticsearch+json; compatible-with=7"
         },
     )
-    try:
-        await elasticsearch.info()
-        await setup_elasticsearch_configs(
-            elasticsearch, app.settings["ELASTICSEARCH_PREFIX"]
+
+
+async def check_elasticsearch(app: Application) -> None:
+    """Check Elasticsearch."""
+    # pylint: disable=invalid-name
+    while True:  # pylint: disable=while-used
+        es: AsyncElasticsearch = cast(
+            AsyncElasticsearch, app.settings.get("ELASTICSEARCH")
         )
-    except Exception as exc:  # pylint: disable=broad-except
-        if raise_exc:
-            raise
-        logger.exception(exc)
-        logger.error("Connecting to Elasticsearch failed!")
-    else:
-        app.settings["ELASTICSEARCH"] = elasticsearch
+        try:
+            await es.info()
+        except Exception as exc:  # pylint: disable=broad-except
+            EVENT_ELASTICSEARCH.clear()
+            logger.exception(exc)
+            logger.error("Connecting to Elasticsearch failed!")
+        else:
+            if not EVENT_ELASTICSEARCH.is_set():
+                await setup_elasticsearch_configs(
+                    es, app.settings["ELASTICSEARCH_PREFIX"]
+                )
+            EVENT_ELASTICSEARCH.set()
+        await asyncio.sleep(20)
 
 
 async def setup_elasticsearch_configs(  # noqa: C901
@@ -533,9 +559,12 @@ async def setup_elasticsearch_configs(  # noqa: C901
                 )
 
 
-async def setup_redis(app: Application, raise_exc: bool = True) -> None:
+def setup_redis(app: Application) -> None:
     """Setup Redis."""  # noqa: D401
     config = app.settings["CONFIG"]
+    if not config.getboolean("REDIS", "ENABLED", fallback=False):
+        app.settings["REDIS"] = None
+        return
 
     kwargs = {
         "db": config.getint("REDIS", "DB", fallback=0),
@@ -583,19 +612,24 @@ async def setup_redis(app: Application, raise_exc: bool = True) -> None:
                 }
             )
 
-    redis = Redis(connection_pool=BlockingConnectionPool(**kwargs))
-    try:
-        await redis.ping()
-    except Exception as exc:  # pylint: disable=broad-except
-        if raise_exc:
-            raise
-        logger.exception(exc)
-        logger.error("Connecting to Redis failed!")
-    else:
-        old_redis: None | Redis = app.settings.get("REDIS")
-        app.settings["REDIS"] = redis
-        if old_redis:
-            await old_redis.close(close_connection_pool=True)
+    app.settings["REDIS"] = Redis(
+        connection_pool=BlockingConnectionPool(**kwargs)
+    )
+
+
+async def check_redis(app: Application) -> None:
+    """Check Redis."""
+    while True:  # pylint: disable=while-used
+        redis: Redis = cast(Redis, app.settings.get("REDIS"))  # type: ignore[type-arg]
+        try:
+            await redis.ping()
+        except Exception as exc:  # pylint: disable=broad-except
+            EVENT_REDIS.clear()
+            logger.exception(exc)
+            logger.error("Connecting to Redis failed!")
+        else:
+            EVENT_REDIS.set()
+        await asyncio.sleep(20)
 
 
 def cancel_all_tasks(loop: asyncio.AbstractEventLoop) -> None:
@@ -622,19 +656,44 @@ def cancel_all_tasks(loop: asyncio.AbstractEventLoop) -> None:
             )
 
 
-def main() -> None | int | str:
-    # pylint: disable=import-outside-toplevel, unused-variable
+async def wait_for_shutdown() -> None:
+    """Wait for the shutdown event."""
+    loop = asyncio.get_running_loop()
+    while not EVENT_SHUTDOWN.is_set():  # pylint: disable=while-used
+        await asyncio.sleep(0.1)
+    loop.stop()
+
+
+def signal_handler(  # noqa: D103
+    signalnum: int, frame: None | types.FrameType
+) -> None:
+    # pylint: disable=unused-argument, missing-function-docstring
+    if signalnum in {signal.SIGINT, signal.SIGTERM}:
+        EVENT_SHUTDOWN.set()
+
+
+def main() -> None | int | str:  # noqa: C901
+    # pylint: disable=too-complex, too-many-branches
+    # pylint: disable=too-many-locals, too-many-statements
     """
     Start everything.
 
     This is the main function that is called when running this file.
     """
+    signal.signal(signal.SIGINT, signal_handler)
+    signal.signal(signal.SIGTERM, signal_handler)
+
     config = configparser.ConfigParser(interpolation=None)
     config.read("config.ini", encoding="utf-8")
 
     setup_logging(config)
 
-    logger.warning("Starting %s %s", NAME, VERSION)
+    logger.info("Starting %s %s", NAME, VERSION)
+
+    if sys.platform == "win32":
+        logger.warning(
+            "Please note that running on Windows is not officially supported."
+        )
 
     # read ignored modules from the config
     for module_name in config.get(
@@ -649,39 +708,94 @@ def main() -> None | int | str:
         return app
 
     apply_config_to_app(app, config)
+    setup_elasticsearch(app)
+    setup_app_search(app)
+    setup_redis(app)
+    setup_apm(app)
 
     behind_proxy = config.getboolean("GENERAL", "BEHIND_PROXY", fallback=False)
 
-    port: int = config.getint("GENERAL", "PORT", fallback=8080)
-    server = app.listen(
-        port,
-        "localhost" if behind_proxy else "",
+    server = HTTPServer(
+        app,
         ssl_options=get_ssl_context(config),
         decompress_request=True,
         xheaders=behind_proxy,
     )
-    app.settings["PORT"] = port
 
-    setup_apm(app)
-    setup_app_search(app)
+    port = config.getint(
+        "GENERAL", "PORT", fallback=8888 if CONTAINERIZED else None
+    )
+
+    sockets = []
+
+    if port:
+        sockets.extend(
+            tornado.netutil.bind_sockets(
+                port, "localhost" if behind_proxy else ""
+            )
+        )
+
+    unix_socket_path = config.get(
+        "GENERAL",
+        "UNIX_SOCKET_PATH",
+        fallback="/data" if CONTAINERIZED else None,
+    )
+
+    if unix_socket_path:
+        os.makedirs(unix_socket_path, exist_ok=True)
+        sockets.append(
+            tornado.netutil.bind_unix_socket(
+                os.path.join(unix_socket_path, f"{NAME}.sock")
+            )
+        )
+
+    processes = config.getint(
+        "GENERAL",
+        "PROCESSES",
+        fallback=0
+        if sys.flags.dev_mode
+        else (tornado.process.cpu_count() if hasattr(os, "fork") else 0),
+    )
+
+    if processes > 0:
+        tornado.process.fork_processes(processes)
+        # yeet all children (there should be none, but do it regardless, just in case)
+        process._children.clear()  # type: ignore[attr-defined]  # pylint: disable=protected-access  # noqa: B950
+        del AUTHORS_CACHE.control.created_by_ultra  # type: ignore[attr-defined]
+        del QUOTES_CACHE.control.created_by_ultra  # type: ignore[attr-defined]
+        del WRONG_QUOTES_CACHE.control.created_by_ultra  # type: ignore[attr-defined]
+
+    task_id = tornado.process.task_id()
+
+    if unix_socket_path and task_id is not None:
+        sockets.append(
+            tornado.netutil.bind_unix_socket(
+                os.path.join(unix_socket_path, f"{NAME}.{task_id}.sock")
+            )
+        )
+
+    server.add_sockets(sockets)
+
+    # pylint: disable=import-outside-toplevel, unused-variable
 
     loop = asyncio.get_event_loop_policy().get_event_loop()
+    wait_for_shutdown_task = loop.create_task(wait_for_shutdown())  # noqa: F841
 
-    # fmt: off
-    setup_es_task = loop.create_task(setup_elasticsearch(app, False))  # noqa: F841
-    setup_redis_task = loop.create_task(setup_redis(app, False))
-    # fmt: on
+    if config.getboolean("ELASTICSEARCH", "ENABLED", fallback=False):
+        check_es_task = loop.create_task(check_elasticsearch(app))  # noqa: F841
+
+    if config.getboolean("REDIS", "ENABLED", fallback=False):
+        check_redis_task = loop.create_task(check_redis(app))  # noqa: F841
 
     from .quotes import update_cache_periodically
 
-    quotes_cache_update_task = loop.create_task(  # noqa: F841
-        update_cache_periodically(app, setup_redis_task)
-    )
+    if not task_id:
+        quotes_cache_update_task = loop.create_task(  # noqa: F841
+            update_cache_periodically(app)
+        )
 
     try:
         loop.run_forever()
-    except KeyboardInterrupt:
-        pass
     finally:
         try:
             server.stop()
@@ -696,4 +810,5 @@ def main() -> None | int | str:
                 loop.run_until_complete(loop.shutdown_default_executor())
             finally:
                 loop.close()
+
     return None
