@@ -1,4 +1,4 @@
-// @license magnet:?xt=urn:btih:0b31508aeb0634b347b8270c7bee4d411b5d4109&dn=agpl-3.0.txt AGPL-3.0
+// @license magnet:?xt=urn:btih:0b31508aeb0634b347b8270c7bee4d411b5d4109&dn=agpl-3.0.txt GNU-AGPL-3.0-or-later
 const bodyDiv = elById("body");
 
 function getJSONURLWithParams(originalUrl) {
@@ -40,6 +40,7 @@ function dynLoadOnData(data, onpopstate) {
             data["title"],
             url
         );
+        w.lastLocation = url;
     }
     if (!data["body"]) {
         w.location.reload();
@@ -77,16 +78,15 @@ function dynLoadOnData(data, onpopstate) {
         }
     }
 
-    const title = data["title"];
-    d.title = title;
-    const shortTitle = data["short_title"] || title;
-    let titleStyleText = `#title:before{content:"${shortTitle}"}`;
-    if (shortTitle !== title)
-        titleStyleText += (
-            `@media(min-width:500px){#title:before{content:"${title}"}}`
-        );
+    if (w.hideSitePane) hideSitePane();
+
+    d.title = data["title"];
+    const titleElement = elById("title");
+    titleElement.setAttribute(
+        "short_title", data["short_title"] || data["title"]
+    );
+    titleElement.innerText = data["title"];
 
-    elById("title-style").innerText = titleStyleText;
     dynLoadReplaceAnchors();
     w.urlData = data;
     return true;
@@ -156,7 +156,8 @@ function dynLoad(url) {
 
 function dynLoadSwitchToURL(url, allowSameUrl = false) {
     if (!allowSameUrl && url === w.location.href) {
-        error("URL is the same as current, ignoring");
+        log("URL is the same as current, just hide site pane");
+        if (w.hideSitePane) hideSitePane();
         return;
     }
     bodyDiv.prepend(
@@ -170,9 +171,9 @@ function dynLoadSwitchToURL(url, allowSameUrl = false) {
         (error) => {
             log(error);
             if (url === w.location.href) {
-                w.location.href = url;
-            } else {
                 w.location.reload();
+            } else {
+                w.location.href = url;
             }
         }
     );
